# (c) 2012-2018 Dativa, all rights reserved
# -----------------------------------------
# Usage subject to license agreement
# hello@dativa.com for more information

from dativa.scrubber.tests import _BaseTest
from dativa.scrubber import ScrubberValidationError


class StringTests(_BaseTest):

    def test_to_text(self):
        bucket = "test-bucket"

        business_rules = [
            {'rule_type': 'Date',
             'field': 'timestamp',
             'params': {'fallback_mode': 'remove_record',
                        'skip_blank': False,
                        'date_format': '%Y-%m-%d %H:%M:%S'}},
            {'rule_type': 'String',
             'field': 'discount',
             'params': {'fallback_mode': 'remove_record',
                        'minimum_length': 0,
                        'maximum_length': 1}},
            {'rule_type': 'String',
             'field': 'custState',
             'params': {'fallback_mode': 'remove_record',
                        'minimum_length': 0,
                        'maximum_length': 2,
                        'skip_blank': True,
                        'regex': '(?!NY|CA)'}},
            {'rule_type': 'String',
             'field': 'expressConsent',
             'params': {'fallback_mode': 'remove_record',
                        'minimum_length': 4,
                        'skip_blank': True,
                        'maximum_length': 4,
                        'regex': '(T|t)rue'}},
            {'rule_type': 'Number',
             'field': 'latitude',
             'params': {'fallback_mode': 'remove_record',
                        'fix_decimal_places': True,
                        'decimal_places': 7,
                        'minimum_value': -90,
                        'maximum_value': 90}},
            {'rule_type': 'Number',
             'field': 'longitude',
             'params': {'fallback_mode': 'remove_record',
                        'fix_decimal_places': True,
                        'decimal_places': 7,
                        'minimum_value': -180,
                        'maximum_value': 180}},
            {'rule_type': 'String',
             'field': 'custCountry',
             'params': {'maximum_length': 3,
                        'minimum_length': 2,
                        'regex': 'US.?',
                        'fallback_mode': 'remove_record',
                        'skip_blank': True}},

            {'rule_type': 'String',
             'field': 'vin',
             'params': {'maximum_length': 17,
                        'minimum_length': 17,
                        'regex': '.{12}\d{5}',
                        'fallback_mode': 'remove_record',
                        }},
        ]
        source_params = {'minimum_length': 2,
                         'maximum_length': 5,
                         'regex': 'IDS2|GM|TOY|TOYO',
                         'fallback_mode': 'remove_record'}

        source_params_blanks = {'minimum_length': 2,
                                'maximum_length': 5,
                                'skip_blank': True,
                                'regex': 'IDS2|GM|TOY|ZCW',
                                'fallback_mode': 'remove_record'}

        date_param_blank = {'fallback_mode': 'remove_record',
                            'skip_blank': True,
                            'date_format': '%Y-%m-%d %H:%M:%S'}

        cleaning_rules = [
            {'rule_type': 'String',
             'field': 'engineOff',
             'params': {'minimum_length': 0,
                        'maximum_length': 10,
                        'regex': 'True|False',
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'Number',
             'field': 'fuelCapacity',
             'params': {'minimum_value': 0,
                        'maximum_value': 200,
                        'fallback_mode': 'remove_record',
                        'decimal_places': 1,
                        'fix_decimal_places': True}},
            {'rule_type': 'String',
             'field': 'fuelUom',
             'params': {'minimum_length': 0,
                        'maximum_length': 1024,
                        'regex': 'gal',
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'gpsCountry',
             'params': {'minimum_length': 0,
                        'maximum_length': 25,
                        'fallback_mode': 'remove_record',
                        'skip_blank': True}},
            {'rule_type': 'Date',
             'field': 'lastCommDate',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'lastUpdate',
             'params': date_param_blank},
            {'rule_type': 'String',
             'field': 'licenseCountry',
             'params': {'fallback_mode': 'remove_record',
                        'minimum_length': 2,
                        'maximum_length': 2}},
            {'rule_type': 'String',
             'field': 'make',
             'params': {'fallback_mode': 'remove_record',
                        'maximum_length': 4,
                        'minimum_length': 2}},
            {'rule_type': 'String',
             'field': 'model',
             'params': {'fallback_mode': 'remove_record',
                        'maximum_length': 4,
                        'minimum_length': 4}},
            {'rule_type': 'String',
             'field': 'mva',
             'params': {'fallback_mode': 'remove_record',
                        'maximum_length': 10,
                        'minimum_length': 8}},
            {'rule_type': 'String',
             'field': 'odometerUom',
             'params': {'maximum_length': 2,
                        'minimum_length': 2,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'Date',
             'field': 'parkedTime',
             'params': date_param_blank},
            {'rule_type': 'String',
             'field': 'source',
             'params': source_params},
            {'rule_type': 'String',
             'field': 'tcu',
             'params': source_params},
            {'rule_type': 'Number',
             'field': 'year',
             'params': {'maximum_value': 2030,
                        'minimum_value': 2000,
                        'fallback_mode': 'remove_record',
                        'decimal_places': 0,
                        'fix_decimal_places': False}},
            {'rule_type': 'String',
             'field': 'source_fuelMetrics',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'source_geoPoint',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'source_odometerMetrics',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_tireMetrics',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_fuelMetrics',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_tcuData',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_geoPoint',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_odometerMetrics',
             'params': source_params_blanks},
            {'rule_type': 'String',
             'field': 'tcu_rentalData',
             'params': source_params_blanks},
            {'rule_type': 'Date',
             'field': 'timestamp_tireMetrics',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'timestamp_fuelMetrics',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'timestamp_geoPoint',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'timestamp_odometerMetrics',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'timestamp_rentalData',
             'params': date_param_blank},
            {'rule_type': 'Date',
             'field': 'checkInDue',
             'params': date_param_blank},
            {'rule_type': 'Number',
             'field': 'fuel',
             'params': {'maximum_value': 200,
                        'minimum_value': 0,
                        'decimal_places': 2,
                        'fix_decimal_places': True,
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'Number',
             'field': 'fuelInEights',
             'params': {'maximum_value': 8,
                        'minimum_value': 0,
                        'skip_blank': True,
                        'decimal_places': 0,
                        'fix_decimal_places': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'Number',
             'field': 'fuelInTenths',
             'params': {'maximum_value': 200,
                        'minimum_value': 0,
                        'decimal_places': 1,
                        'skip_blank': True,
                        'fix_decimal_places': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'uom_fuelMetrics',
             'params': {'maximum_length': 3,
                        'minimum_length': 1,
                        'skip_blank': True,
                        'fallback_mode': 'remove_record',
                        'regex': 'gal|l'}},
            {'rule_type': 'Number',
             'field': 'odometer',
             'params': {'maximum_value': 200000,
                        'minimum_value': 0,
                        'decimal_places': 1,
                        'fix_decimal_places': True,
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'uom_odometerMetrics',
             'params': {'maximum_length': 2,
                        'minimum_length': 0,
                        'fallback_mode': 'remove_record',
                        'skip_blank': True,
                        'regex': 'mi|km'}},
            {'rule_type': 'String',
             'field': 'brand',
             'params': {'maximum_length': 1,
                        'minimum_length': 0,
                        'regex': 'A|B|C|P',
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'ciLocIdentifier',
             'params': {'maximum_length': 1,
                        'minimum_length': 0,
                        'regex': 'C|A|L',
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'coLocIdentifier',
             'params': {'maximum_length': 1,
                        'minimum_length': 0,
                        'regex': 'C|A|L',
                        'skip_blank': True,
                        'fallback_mode': 'remove_record'}},
            {'rule_type': 'String',
             'field': 'rentalStatus',
             'params': {'maximum_length': 6,
                        'minimum_length': 6,
                        'fallback_mode': 'remove_record',
                        'skip_blank': True}}
        ]

        config = dict()

        config['rules'] = business_rules + cleaning_rules

        scrubber = self.ScrubberClass()

        self.maxDiff=None
        self.assertEquals(scrubber.to_text(config),
                          """field,type,date_format,fallback_mode,minimum_length,maximum_length,regex,skip_blank,decimal_places,minimum_value,maximum_value,fix_decimal_places
timestamp,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,,,,,
discount,String,,remove_record,0.0,1.0,,,,,,
custState,String,,remove_record,0.0,2.0,(?!NY|CA),True,,,,
expressConsent,String,,remove_record,4.0,4.0,(T|t)rue,True,,,,
latitude,Number,,remove_record,,,,,7.0,-90.0,90.0,True
longitude,Number,,remove_record,,,,,7.0,-180.0,180.0,True
custCountry,String,,remove_record,2.0,3.0,US.?,True,,,,
vin,String,,remove_record,17.0,17.0,.{12}\\d{5},,,,,
engineOff,String,,remove_record,0.0,10.0,True|False,True,,,,
fuelCapacity,Number,,remove_record,,,,,1.0,0.0,200.0,True
fuelUom,String,,remove_record,0.0,1024.0,gal,,,,,
gpsCountry,String,,remove_record,0.0,25.0,,True,,,,
lastCommDate,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
lastUpdate,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
licenseCountry,String,,remove_record,2.0,2.0,,,,,,
make,String,,remove_record,2.0,4.0,,,,,,
model,String,,remove_record,4.0,4.0,,,,,,
mva,String,,remove_record,8.0,10.0,,,,,,
odometerUom,String,,remove_record,2.0,2.0,,,,,,
parkedTime,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
source,String,,remove_record,2.0,5.0,IDS2|GM|TOY|TOYO,,,,,
tcu,String,,remove_record,2.0,5.0,IDS2|GM|TOY|TOYO,,,,,
year,Number,,remove_record,,,,,0.0,2000.0,2030.0,False
source_fuelMetrics,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
source_geoPoint,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
source_odometerMetrics,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_tireMetrics,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_fuelMetrics,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_tcuData,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_geoPoint,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_odometerMetrics,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
tcu_rentalData,String,,remove_record,2.0,5.0,IDS2|GM|TOY|ZCW,True,,,,
timestamp_tireMetrics,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
timestamp_fuelMetrics,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
timestamp_geoPoint,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
timestamp_odometerMetrics,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
timestamp_rentalData,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
checkInDue,Date,%Y-%m-%d %H:%M:%S,remove_record,,,,True,,,,
fuel,Number,,remove_record,,,,True,2.0,0.0,200.0,True
fuelInEights,Number,,remove_record,,,,True,0.0,0.0,8.0,True
fuelInTenths,Number,,remove_record,,,,True,1.0,0.0,200.0,True
uom_fuelMetrics,String,,remove_record,1.0,3.0,gal|l,True,,,,
odometer,Number,,remove_record,,,,True,1.0,0.0,200000.0,True
uom_odometerMetrics,String,,remove_record,0.0,2.0,mi|km,True,,,,
brand,String,,remove_record,0.0,1.0,A|B|C|P,True,,,,
ciLocIdentifier,String,,remove_record,0.0,1.0,C|A|L,True,,,,
coLocIdentifier,String,,remove_record,0.0,1.0,C|A|L,True,,,,
rentalStatus,String,,remove_record,6.0,6.0,,True,,,,
""")



# (c) 2012-2018 Dativa, all rights reserved
# -----------------------------------------
#  This code is licensed under MIT license (see license.txt for details)

import datetime
import json
import os

import pandas as pd
from newtools import PandasDoggo
from newtools import PersistentFieldLogger


class ReportEntry:
    """
    An individual report entry.

    Parameters
    ----------
    rule: the name of the rule that produced this entry
    field: the field being processed when the entry was produced
    df: the DataFrame containing the invalid records
    category: the type of entry, one of:
        HistoryCategory.IGNORED - the records were not altered
        HistoryCategory.REPLACED - the records were replaced with other values
        HistoryCategory.REMOVED - the records have been removed
        HistoryCategory.MODIFIED - the records have been modified
        HistoryCategory.INSERTED - new records have been inserted
    description: a textual description of the changes made (if any)
    """

    date = ""
    field = ""
    df = None
    category = ""
    description = ""
    number_records = ""

    def __init__(self, rule, field, df, category, description):
        self.date = datetime.datetime.now()
        self.field = field
        self.df = pd.DataFrame(df)
        self.number_records = df.shape[0]
        self.category = category
        self.description = description
        self.rule = rule

    def __str__(self):
        return "{0}, Field {1}({2}): #{3} {4}/|{5}".format(self.date,
                                                           self.field,
                                                           self.rule,
                                                           self.number_records,
                                                           self.category,
                                                           self.description)

    def get_log_dict(self):
        """
        Returns a dict containing the log entries
        """
        return {"date": self.date,
                "field": self.field,
                "rule": self.rule,
                "number_records": self.number_records,
                "category": self.category,
                "description": self.description}

    def get_records(self):
        """
        returns the DataFrame containing the values
        """
        return self.df


class DefaultReportWriter:
    """
    A default implementation of the ReportWriter that stores ReportEntries
    and returns them.
    """

    report = []

    def reset_report(self):
        self.report = []

    def log_history(self,
                    rule,
                    field,
                    df,
                    category,
                    description
                    ):
        """
        Logs a report entry
        """

        assert isinstance(df, pd.DataFrame), "df passed to log_history is not a DataFrame"

        self.report.append(ReportEntry(rule=rule,
                                       field=field,
                                       df=df,
                                       category=category,
                                       description=description
                                       ))

        return

    def get_report(self):
        """
        Returns an array of all ReportEntries created
        """
        return self.report


class LoggingReportWriter(DefaultReportWriter):
    """
    A class to log all reports to the python log file
    """

    def __init__(self, logger, quarantine_path, error_severities=None, warning_severities=None):
        self.logger = logger
        self.csv_path = quarantine_path
        self.error_severities = [] if error_severities is None else error_severities
        self.warning_severities = [] if warning_severities is None else warning_severities

    def log_history(self,
                    rule,
                    field,
                    df,
                    category,
                    description
                    ):
        """
        Logs out the report details and saves the dataframe to the specified path
        """

        if df is not None and df.shape[0] > 0:
            filename = "{field}-{rule}-{date:%Y%m%d%H%M%S}.csv".format(
                field=field,
                rule=rule,
                date=datetime.datetime.now()).lower()
            PandasDoggo().save_csv(df, os.path.join(self.csv_path, filename))

            if type(self.logger) is PersistentFieldLogger:
                if category in self.error_severities:
                    self.logger.error(category=category,
                                      field=field,
                                      description=description,
                                      filename=filename,
                                      number=df.shape[0])
                elif category in self.warning_severities:
                    self.logger.warning(category=category,
                                        field=field,
                                        description=description,
                                        filename=filename,
                                        number=df.shape[0])
                else:
                    self.logger.info(category=category,
                                     field=field,
                                     description=description,
                                     filename=filename,
                                     number=df.shape[0])
            else:
                message = "{category}: #{number} from {field} : {description} [{filename}]".format(
                    category=category,
                    field=field,
                    description=description,
                    filename=filename,
                    number=df.shape[0]
                )

                if category in self.error_severities:
                    self.logger.error(message)
                elif category in self.warning_severities:
                    self.logger.warning(message)
                else:
                    self.logger.info(message)



# (c) 2012-2018 Dativa, all rights reserved
# -----------------------------------------
# Usage subject to license agreement
# hello@dativa.com for more information

import sys
import logging
import os
import pandas as pd
import numpy as np
from newtools import CSVDoggo
from newtools.tests.base_test import BaseTest
from dativa.scrubber import Scrubber

logger = logging.getLogger("dativa.scrubber.tests")


class _BaseTest(BaseTest):
    create_test_data = False

    test_dir = os.path.join(os.path.dirname(__file__), "test-data")
    ScrubberClass = Scrubber

    def _compare_df_to_file(self, csv, df, clean_file):
        """

        :param csv: an instance of a CSVHandler class
        :param df: the dataframe to compare
        :param clean_file: the file to compare
        :return: raises an exception if they not the same
        """

        if self.create_test_data:
            csv.save_df(df, clean_file)

        print(clean_file)

        pd.testing.assert_frame_equal(csv.load_df(clean_file,
                                                  dtype=str,
                                                  na_values='',
                                                  skip_blank_lines=False,
                                                  keep_default_na=False).fillna("N/A").reset_index(drop=True),
                                      df.reset_index(drop=True).replace('', pd.NA).fillna("N/A").astype(str),
                                      check_like=True,
                                      check_dtype=False)

    def _test_filter_actual(self,
                            dirty_file,
                            clean_file,
                            config,
                            expected_report,
                            encoding,
                            delimiter,
                            quotechar,
                            report_writer,
                            profile,
                            expected_df_dict,
                            dirty_df):

        csv = CSVDoggo(encoding=encoding,
                       delimiter=delimiter,
                       quotechar=quotechar,
                       base_path=self.test_dir,
                       line_terminator="\n")

        scrubber = self.ScrubberClass(report_writer, profile)

        dataframe_dict = {}
        for file in scrubber.get_files_from_config(config):
            dataframe_dict[file] = csv.load_df(file, force_dtype=np.str, header=None, skip_blank_lines=False)
        if isinstance(dirty_df, pd.DataFrame):
            df = dirty_df
        else:
            print(dirty_file)
            df = csv.load_df(dirty_file, skip_blank_lines=False)

        report = scrubber.run(df, config, dataframe_dict)

        print(",")
        print("report = [")
        np.set_printoptions(threshold=sys.maxsize)
        for r in report:
            print("{")
            print("""
            'field': '{0}',
                   'rule': '{1}',
                   'number_records': {2},
                   'category': '{3}',
                   'description': '{4}',
                   'df': {5},
                   """.format(r.field, r.rule, r.number_records, r.category, r.description,
                              repr(r.df.values).replace("array(", "").replace(", dtype=object)", "").replace(
                                  "dtype=object),", "").replace("Timestamp", "_to_time")).replace("nan", "None"))
            print("},")
        print("]")

        self._compare_df_to_file(csv, df, clean_file)

        if expected_report is not None:

            self.assertEqual(len(report), len(expected_report))
            for i in range(0, len(report)):
                self.assertEqual(report[i].field, expected_report[i]["field"])
                self.assertEqual(report[i].rule, expected_report[i]["rule"])
                self.assertEqual(report[i].number_records, expected_report[i]["number_records"])
                self.assertEqual(report[i].category, expected_report[i]["category"])
                self.assertEqual(report[i].description, expected_report[i]["description"])
                np.array_equal(report[i].df.values, expected_report[i]["df"], )

        if expected_df_dict is not None:
            for file in expected_df_dict:
                try:
                    expected_df = csv.load_df(expected_df_dict[file])
                    actual_df = dataframe_dict[file]
                    pd.testing.assert_frame_equal(expected_df, actual_df)
                except TypeError:
                    expected_df = csv.load_df(file)
                    actual_df = dataframe_dict[file]
                    pd.testing.assert_frame_equal(expected_df, actual_df)

    def _test_filter(self,
                     dirty_file,
                     clean_file,
                     config,
                     report=None,
                     encoding="UTF-8",
                     delimiter=",",
                     quotechar='"',
                     report_writer=None,
                     profile=None,
                     expected_error=None,
                     expected_df_dict=None,
                     dirty_df=False):

        logger.debug("testing {0}".format(dirty_file))

        if expected_error is None:
            self._test_filter_actual(dirty_file,
                                     clean_file,
                                     config,
                                     report,
                                     encoding,
                                     delimiter,
                                     quotechar,
                                     report_writer,
                                     profile,
                                     expected_df_dict=expected_df_dict,
                                     dirty_df=dirty_df
                                     )

            if report is None:
                raise ValueError("Report not set for test")

            return

        with self.assertRaises(expected_error):
            return self._test_filter_actual(dirty_file,
                                            clean_file,
                                            config,
                                            report,
                                            encoding,
                                            delimiter,
                                            quotechar,
                                            report_writer,
                                            profile,
                                            expected_df_dict=expected_df_dict,
                                            dirty_df=dirty_df)



# (c) 2012-2018 Dativa, all rights reserved
# -----------------------------------------
#  This code is licensed under MIT license (see license.txt for details)

"""
A set of functions to improve dataframe column handling in pandas
"""

import numpy as np
import pandas as pd
from pandas._libs.tslibs.strptime import array_strptime
from pandas._libs import tslib
from pandas.core.dtypes.generic import ABCSeries

try:
    from pandas.core.dtypes.generic import ABCIndexClass
except ImportError:  # pragma: no cover
    # in pandas v1.3 this class was renamed
    from pandas.core.dtypes.generic import ABCIndex as ABCIndexClass

from pandas.core.dtypes.common import is_list_like
from pandas import Series, DatetimeIndex
from datetime import datetime


def is_numeric(series_or_string):
    """
    Returns whether the series or string isnumeric
    """
    if not pd:  # pragma: no cover
        raise ImportError("pandas must be installed to run is_numeric")

    if isinstance(series_or_string, pd.Series):
        return np.isreal(series_or_string).all()
    else:
        try:
            float(series_or_string)
            return True
        except ValueError:
            return False


def get_column_name(df, column, total_columns=None):
    """
    gets the name of a column from either an integer or a passed name.
    Allows the total columns to be overwritten for cases where additional columns
    are added to the dataframe.
    """
    if total_columns is None:
        total_columns = df.shape[1]

    if column in df.columns:
        return column

    try:
        column_index = int(column)
        if column_index < 0:
            column_index = column_index + total_columns
        return df.columns.values.tolist()[column_index]
    except (ValueError, TypeError):  # we should be testing this?
        raise KeyError('Invalid column name: {0} not in {1}'.format(column, df.columns))


def get_unique_column_name(df, name, suffix='_clean'):
    """
    Gets a unique name for a new column
    """

    if name not in df.columns:
        return name
    else:
        return get_unique_column_name(df, str(name) + suffix)


def string_to_datetime(series_or_string, format, errors="raise"):
    """
    Converts a series (or string) to a timestamp according
    to the passed format string.
    """
    pd_old_version = "0.23.4"

    if format == "%s":
        # Please remove the no cover this as soon as possible and add tests to cover this
        if errors == "raise" and not is_numeric(series_or_string):  # pragma: no cover
            raise ValueError()
        else:
            return pd.to_datetime(series_or_string, unit='s', errors=errors)
    else:
        # hard coded subset of pd.to_datetime to prevent automatic recognition of ISO8601 format
        # Please remove the no cover this as soon as possible and add tests to cover this
        if isinstance(series_or_string, tslib.Timestamp): # pragma: no cover
            return DatetimeIndex(series_or_string)
        elif isinstance(series_or_string, ABCSeries):
            values = array_strptime(series_or_string._values, format, exact=True, errors=errors)
            if pd.__version__ > pd_old_version:
                values = values[0]
            return Series(DatetimeIndex(values), index=series_or_string.index, name=series_or_string.name)
        # Please remove the no cover this as soon as possible and add tests to cover this
        elif isinstance(series_or_string, ABCIndexClass):  # pragma: no cover
            values = array_strptime(series_or_string._values, format, exact=True, errors=errors)
            if pd.__version__ > pd_old_version:
                values = values[0]
            return DatetimeIndex(values)
        # Please remove the no cover this as soon as possible and add tests to cover this
        elif is_list_like(series_or_string):  # pragma: no cover
            values = array_strptime(series_or_string._values, format, exact=True, errors=errors)
            if pd.__version__ > pd_old_version:
                values = values[0]
            return DatetimeIndex(values)
        else:
            values = array_strptime(pd.Series([series_or_string]).values, format, exact=True, errors=errors)
            if pd.__version__ > pd_old_version:
                values = values[0]
            return DatetimeIndex(values)[0]


def _datetime_to_string(timestamp, format):
    try:
        if format == "%s":
            return "{0:.0f}".format((timestamp - datetime(1970, 1, 1)).total_seconds())
        else:
            return timestamp.strftime(format)
    except ValueError:
        return None


def datetime_to_string(timestamp, format):
    """
    Takes an individual timestamp and returns a string according to
    the passed format
    """
    if not pd:  # pragma: no cover
        raise ImportError("pandas must be installed to run datetime_to_string")

    if type(timestamp) is pd.Series:
        return timestamp.apply(lambda x: _datetime_to_string(x, format))
    else:
        return _datetime_to_string(timestamp, format)


def format_string_is_valid(format):
    """
    Checks a format string to see whether it returns anything date like
    """
    if not pd:  # pragma: no cover
        raise ImportError("pandas must be installed to run format_string_is_valid")

    date1 = datetime(1, 2, 3, 4, 5, 6, 7)
    date2 = datetime(8, 9, 10, 11, 12, 13, 14)
    if _datetime_to_string(date1, format) == _datetime_to_string(date2, format):
        return False

    return True
